from pathlib import Path

from jinjax import Catalog, Component
from line_profiler import LineProfiler


HERE = Path(__file__).parent

catalog = Catalog()
catalog.add_folder(HERE)

profile = LineProfiler(
    Catalog.irender,
    Catalog._get_from_file,
    Component.__init__,
    Component.from_cache,
    Component.filter_args,
    Component.render,
)

def render_jinjax():
    for _ in range(1000):
        catalog.render("Hello", message="Hey there")


if __name__ == "__main__":
    print("Profiling...")
    profile.runcall(render_jinjax)
    profile.print_stats()



import pytest

import jinjax


def test_add_folder_with_default_prefix():
    catalog = jinjax.Catalog()
    catalog.add_folder("file_path")

    assert "file_path" in catalog.prefixes[""].searchpath


def test_add_folder_with_custom_prefix():
    catalog = jinjax.Catalog()
    catalog.add_folder("file_path", prefix="custom")

    assert "file_path" in catalog.prefixes["custom"].searchpath


def test_add_folder_with_dirty_prefix():
    catalog = jinjax.Catalog()
    catalog.add_folder("file_path", prefix="/custom.")

    assert "/custom." not in catalog.prefixes
    assert "file_path" in catalog.prefixes["custom"].searchpath


def test_add_folders_with_same_prefix():
    catalog = jinjax.Catalog()
    catalog.add_folder("file_path1", prefix="custom")
    catalog.add_folder("file_path2", prefix="custom")

    assert "file_path1" in catalog.prefixes["custom"].searchpath
    assert "file_path2" in catalog.prefixes["custom"].searchpath


def test_add_same_folder_in_same_prefix_does_nothing():
    catalog = jinjax.Catalog()
    catalog.add_folder("file_path", prefix="custom")
    catalog.add_folder("file_path", prefix="custom")

    assert catalog.prefixes["custom"].searchpath.count("file_path") == 1


def test_add_module_legacy():
    class Module:
        components_path = "legacy_path"
        prefix = "legacy"

    catalog = jinjax.Catalog()
    module = Module()
    catalog.add_module(module)

    assert "legacy_path" in catalog.prefixes["legacy"].searchpath


def test_add_module_legacy_with_default_prefix():
    class Module:
        components_path = "legacy_path"

    catalog = jinjax.Catalog()
    module = Module()
    catalog.add_module(module)

    assert "legacy_path" in catalog.prefixes[""].searchpath


def test_add_module_legacy_with_custom_prefix():
    class Module:
        components_path = "legacy_path"
        prefix = "legacy"

    catalog = jinjax.Catalog()
    module = Module()
    catalog.add_module(module, prefix="custom")

    assert "legacy" not in catalog.prefixes
    assert "legacy_path" in catalog.prefixes["custom"].searchpath


def test_add_module_fails_with_other_modules():
    class Module:
        pass

    catalog = jinjax.Catalog()
    module = Module()
    with pytest.raises(AttributeError):
        catalog.add_module(module)



import pytest

import jinjax


@pytest.fixture()
def folder(tmp_path):
    d = tmp_path / "components"
    d.mkdir()
    return d


@pytest.fixture()
def folder_t(tmp_path):
    d = tmp_path / "templates"
    d.mkdir()
    return d


@pytest.fixture()
def catalog(folder):
    catalog = jinjax.Catalog(auto_reload=False)
    catalog.add_folder(folder)
    return catalog



import typing as t
from pathlib import Path

import jinjax


def application(environ, start_response) -> list[bytes]:
    status = "200 OK"
    headers = [("Content-type", "text/plain")]
    start_response(status, headers)
    return [b"NOPE"]


def make_environ(**kw) -> dict[str, t.Any]:
    kw.setdefault("PATH_INFO", "/")
    kw.setdefault("REQUEST_METHOD", "GET")
    return kw


def mock_start_response(status: str, headers: dict[str, t.Any]):
    pass


def get_catalog(folder: "str | Path", **kw) -> jinjax.Catalog:
    catalog = jinjax.Catalog(**kw)
    catalog.add_folder(folder)
    return catalog


TMiddleware = t.Callable[
    [
        dict[str, t.Any],
        t.Callable[[str, dict[str, t.Any]], None],
    ],
    t.Any
]

def run_middleware(middleware: TMiddleware, url: str):
    return middleware(make_environ(PATH_INFO=url), mock_start_response)


# Tests


def test_css_is_returned(folder):
    (folder / "page.css").write_text("/* Page.css */")
    catalog = get_catalog(folder)
    middleware = catalog.get_middleware(application)

    resp = run_middleware(middleware, "/static/components/page.css")
    assert resp and not isinstance(resp, list)
    text = resp.filelike.read().strip()
    assert text == b"/* Page.css */"


def test_js_is_returned(folder):
    (folder / "page.js").write_text("/* Page.js */")
    catalog = get_catalog(folder)
    middleware = catalog.get_middleware(application)

    resp = run_middleware(middleware, "/static/components/page.js")
    assert resp and not isinstance(resp, list)
    text = resp.filelike.read().strip()
    assert text == b"/* Page.js */"


def test_other_file_extensions_ignored(folder):
    (folder / "Page.jinja").write_text("???")
    catalog = get_catalog(folder)
    middleware = catalog.get_middleware(application)
    resp = run_middleware(middleware, "/static/components/Page.jinja")
    assert resp == [b"NOPE"]


def test_add_custom_extensions(folder):
    (folder / "Page.jinja").write_text("???")
    catalog = get_catalog(folder)
    middleware = catalog.get_middleware(application, allowed_ext=[".jinja"])

    resp = run_middleware(middleware, "/static/components/Page.jinja")
    assert resp and not isinstance(resp, list)
    text = resp.filelike.read().strip()
    assert text == b"???"


def test_custom_root_url(folder):
    (folder / "page.css").write_text("/* Page.css */")
    catalog = get_catalog(folder, root_url="/static/co/")
    middleware = catalog.get_middleware(application)

    resp = run_middleware(middleware, "/static/co/page.css")
    assert resp and not isinstance(resp, list)
    text = resp.filelike.read().strip()
    assert text == b"/* Page.css */"


def test_autorefresh_load(folder):
    (folder / "page.css").write_text("/* Page.css */")
    catalog = get_catalog(folder)
    middleware = catalog.get_middleware(application, autorefresh=True)

    resp = run_middleware(middleware, "/static/components/page.css")
    assert resp and not isinstance(resp, list)
    text = resp.filelike.read().strip()
    assert text == b"/* Page.css */"


def test_autorefresh_block(folder):
    (folder / "Page.jinja").write_text("???")
    catalog = get_catalog(folder)
    middleware = catalog.get_middleware(application, autorefresh=True)

    resp = run_middleware(middleware, "/static/components/Page.jinja")
    assert resp == [b"NOPE"]


def test_multiple_folders(tmp_path):
    folder1 = tmp_path / "folder1"
    folder1.mkdir()
    (folder1 / "folder1.css").write_text("folder1")

    folder2 = tmp_path / "folder2"
    folder2.mkdir()
    (folder2 / "folder2.css").write_text("folder2")

    catalog = jinjax.Catalog()
    catalog.add_folder(folder1)
    catalog.add_folder(folder2)
    middleware = catalog.get_middleware(application)

    resp = run_middleware(middleware, "/static/components/folder1.css")
    assert resp.filelike.read() == b"folder1"
    resp = run_middleware(middleware, "/static/components/folder2.css")
    assert resp.filelike.read() == b"folder2"


def test_multiple_folders_precedence(tmp_path):
    folder1 = tmp_path / "folder1"
    folder1.mkdir()
    (folder1 / "name.css").write_text("folder1")

    folder2 = tmp_path / "folder2"
    folder2.mkdir()
    (folder2 / "name.css").write_text("folder2")

    catalog = jinjax.Catalog()
    catalog.add_folder(folder1)
    catalog.add_folder(folder2)
    middleware = catalog.get_middleware(application)

    resp = run_middleware(middleware, "/static/components/name.css")
    assert resp.filelike.read() == b"folder1"



import timeit
from pathlib import Path

from fastapi.templating import Jinja2Templates
from jinja2 import Environment, FileSystemLoader

from jinjax import Catalog


here = Path(__file__).parent
number = 10_000

catalog = Catalog()
catalog.add_folder(here)

env = Environment(loader=FileSystemLoader(here))

templates = Jinja2Templates(directory=here)


def render_jinjax_simple():
    """simple case"""
    catalog.render("Simple", message="Hey there")


def render_jinjax_real():
    """realistic case"""
    catalog.render("Real", message="Hey there")


def render_jinja():
    env.get_template("hello.html").render(message="Hey there")


def render_fastapi():
    templates.TemplateResponse("hello.html", {"request": None, "message": "Hey there"})


def benchmark_no_cache(func):
    print(f"NO CACHE: {number:_} renders of {func.__doc__}...\n")
    catalog.use_cache = False
    benchmark(func)


def benchmark_auto_reload(func):
    print(f"CACHE, AUTO-RELOAD: {number:_} renders of {func.__doc__}...\n")
    catalog.use_cache = True
    catalog.auto_reload = True
    benchmark(func)


def benchmark_no_auto_reload(func):
    print(f"CACHE, NO AUTO-RELOAD: {number:_} renders of {func.__doc__}...\n")
    catalog.use_cache = True
    catalog.auto_reload = False
    benchmark(func)


def benchmark(func):
    time_jinjax = timeit.timeit(func, number=number)
    print_line("JinjaX", time_jinjax)
    print(f"{time_jinjax / time_jinja:.1f} times Jinja")
    print(f"{time_jinjax / time_fastapi:.1f} times FastApi")


def print_line(name, time):
    print(f"{name}: {(time / number):.12f}s per render ({(1_000_000 * time / number):.0f}µs), {time:.1f}s total")


def print_separator():
    print()
    print("-" * 60)


if __name__ == "__main__":
    print("Benchmarking...\n")
    time_jinja = timeit.timeit(render_jinja, number=number)
    time_fastapi = timeit.timeit(render_fastapi, number=number)

    print_line("Jinja", time_jinja)
    print_line("FastApi", time_fastapi)
    print_separator()
    benchmark_no_cache(render_jinjax_simple)
    print_separator()
    benchmark_auto_reload(render_jinjax_simple)
    print_separator()
    benchmark_no_auto_reload(render_jinjax_simple)
    print_separator()
    benchmark_no_cache(render_jinjax_real)
    print_separator()
    benchmark_auto_reload(render_jinjax_real)
    print_separator()
    benchmark_no_auto_reload(render_jinjax_real)
    print()



import logging


logger = logging.getLogger("jinjax")

DELIMITER = "."
SLASH = "/"


def get_url_prefix(prefix: str) -> str:
    url_prefix = (
        prefix.strip().strip(f"{DELIMITER}{SLASH}").replace(DELIMITER, SLASH)
    )
    if url_prefix:
        url_prefix = f"{url_prefix}{SLASH}"
    return url_prefix



import re
import typing as t
from pathlib import Path


try:
    from whitenoise import WhiteNoise
    from whitenoise.responders import Redirect, StaticFile
except ImportError:
    WhiteNoise = object

RX_FINGERPRINT = re.compile("(.*)-([abcdef0-9]{64})")


class ComponentsMiddleware(WhiteNoise):  # type: ignore
    """WSGI middleware for serving components assets"""

    allowed_ext: tuple[str, ...]

    def __init__(self, **kwargs) -> None:
        if WhiteNoise is object:
            raise ImportError(
                "The ComponentsMiddleware requires the package `whitenoise`"
                + " to be installed. \nRun `pip install jinjax[whitenoise]` to do it."
            )
        self.allowed_ext = kwargs.pop("allowed_ext", ())
        super().__init__(**kwargs)

    def find_file(self, url: str) -> "StaticFile | Redirect | None":
        if self.allowed_ext and not url.endswith(self.allowed_ext):
            return None

        # Ignore the fingerprint in the filename
        # since is only for managing the cache in the client
        relpath = Path(url)
        ext = "".join(relpath.suffixes)
        stem = relpath.name.removesuffix(ext)
        fingerprinted = RX_FINGERPRINT.match(stem)
        if fingerprinted:
            stem = fingerprinted.group(1)
            relpath = relpath.with_name(f"{stem}{ext}")

        return super().find_file(str(relpath))

    def add_file_to_dictionary(
        self, url: str, path: str, stat_cache: t.Any = None
    ) -> None:
        if not self.allowed_ext or url.endswith(self.allowed_ext):
            super().add_file_to_dictionary(url, path, stat_cache)
